@@ -56,7 +56,7 @@ dai_token <- function(path = Sys.getenv("GCS_AUTH_FILE"),
   }
 
   if (!inherits(token, "Token2.0")) {
-    message("Invalid GCS credentials. No token produced.")
+    cli::cli_alert_danger("Invalid GCS credentials. No token produced.")
   } else {
     return(token)
   }
@@ -78,8 +78,7 @@ dai_user <- function() {
 
   response <- httr::GET("https://www.googleapis.com/oauth2/v1/userinfo",
                     httr::config(token = dai_token()))
-
-  return(response)
+  httr::content(response)
 
   }
 
@@ -102,10 +101,6 @@ get_project_id <- function(path = Sys.getenv("GCS_AUTH_FILE")) {
     stop("Error: invalid path parameter.")
     }
 
-  json <- jsonlite::fromJSON(path)
-
-  project_id <- json$project_id
-
-  return(project_id)
+  jsonlite::fromJSON(path)$project_id
 
   }
